#ifndef _INTERNALS_H
#define _INTERNALS_H

//include standard header files
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include "jupiter_db.h"

#define IDXLEN_SZ	4
#define SEP			':'
#define SPACE		' '
#define NEWLINE		'\n'

#define PTR_SZ		6		 //size of ptr field in hash chain
#define PTR_MAX		999999   //max file offset = 10^PTR_SZ - 1
#define NHASH_DEF	137		 //default hash table size. Use prime number for hash table performance.
#define FREE_OFF	0		 //free list offset in index file
#define HASH_OFF	PTR_SZ   //hash table offset in index file

typedef unsigned long DBHASH;	//hash values
typedef unsigned long COUNT;	//unsigned counter

//internal db structure
typedef struct {
	int idxfd;	//index file fd
	int datfd;	//data file fd
	int dscfd;	//describe file fd
	char *idxbuf;	//buffer for index record
	char *datbuf;	//buffer for data record
	char *dscbuf;	//buffer for describe record
	char *name;		//name of table
	off_t idxoff;	//offset in index file of index record: key is at idxoff + PTR_SZ + IDXLEN_SZ
	size_t idxlen;  //len of index record: starts after IDXLEN_SZ and includes ending newline

	off_t datoff;	//offset in data file of data record
	size_t datlen;	//length of data record: include ending newline
	off_t ptrval;	//chain ptr contents in index record
	off_t ptroff;	//chain ptr offset pointing to this index record

	off_t chainoff;	//offset of hash chain for this index record
	off_t hashoff;	//offset in index file of hash table

	DBHASH	nhash;	//current hash table size

	tbl_field *fields; //struct that describes fields in each record
	int numfields; //number of fields in each record

	COUNT cnt_delok;	//number of delete OK
	COUNT cnt_delerr;	//number of delete errors
	COUNT cnt_fetchok;	//number of fetch OK
	COUNT cnt_fetcherr;	//number of fetch errors
	COUNT cnt_nextrec;	//count of nextrec calls
	COUNT cnt_insert_append;	//number of insert appends
	COUNT cnt_insert_reuse;		//number of insert reuses
	COUNT cnt_replace_append;	//number of replace appends
	COUNT cnt_replace_ovwrt;	//number of replace overwrites
	COUNT cnt_storerr;  //number of store errors

} DB_TBL;

typedef struct {
	int dbfd;	//main file fd
	char *name; //name under which db was opened
	struct hash_table *tables;
	int table_count;
} DB;

#endif
