#ifndef _JUPITER_DB_H
#define _JUPITER_DB_H

//Implementation limits
#define IDXLEN_MIN	6
#define IDXLEN_MAX	2048
#define DATLEN_MIN	2
#define DATLEN_MAX	2048

typedef void *DBHANDLE;

typedef enum {
	INT,
	LONG,
	FLOAT,	
	STRING,
} field_type;

typedef struct {
	field_type type;
	char *name;
	int pos;
} tbl_field;


//Public interfaces to Jupiter.
/* Open a database. Same arguments as open(). */
DBHANDLE db_open(const char *pathname, int oflag, ...); 
/* Create a new table in the database. */
DBHANDLE db_tbl_create(DBHANDLE db, const char * tablename, tbl_field *, int num_fields); 
/* Insert, replace or do one of either (store) in the specified table. */
int db_store(DBHANDLE db, const char *tablename, const char *key, const char *data, int db_store_flag); 
/* Delete a record in the specified table pointed to by its key. */
int db_delete(DBHANDLE db, const char *tablename, const char *key); 
/* Retreive a record in the specified table pointed to by its key. */
char *db_fetch(DBHANDLE db, const char *tablename, const char *key); 
/* Rewind the position of index file of the given table to its first index record. */
void db_rewind(DBHANDLE db, const char *tablename); 
/* Get next sequential index record in table's index file. If given key is not null,
 * it will copy the record's key value into it. */
char *db_nextrec(DBHANDLE db, const char * tablename, char *key); 
/* Retrieve all the records in the table. Takes a function pointer for a callback function.*/
int db_retrieve(DBHANDLE db, const char *tablename, void *user_data, int (*user_func)(DBHANDLE db, void * user_data, const char *records));
/* Retrieve information about the fields in the records of a table. */
int db_getfieldinfo(DBHANDLE db, const char *tablename, void *user_data, int (*user_func)(DBHANDLE db, void *user_data, int num_fields, tbl_field *));
/* Close the database. */
void db_close(DBHANDLE db); 

//db_store_flag
#define DB_INSERT 	1
#define DB_REPLACE	2
#define DB_STORE	3

#endif
